

#define PLAY_CELL( ptr0, the_player ) \
 \
	play_p = ptr0; \
	*(UBYTE*)*play_p++ = (UBYTE)the_player; \
 \
	for ( play_k = *play_p++; play_k ; play_k-- ) \
		*(UBYTE*)*play_p++ = (UBYTE)the_player;


#define UNPLAY_CELL( ptr0, the_enemy ) \
 \
	play_p = ptr0; \
	*(UBYTE*)*play_p++ = CODE_EMPTY; \
 \
	for ( play_k = *play_p++; play_k ; play_k-- ) \
		*(UBYTE*)*play_p++ = the_enemy;


static int minimax(
	const int depth,
	void *curr_ptr,
	const int prev_pass,
	int black_best_coef, int white_best_coef,
	int the_player, int the_enemy )
	{
		void			*ptr1, *ptr2;
		UINT			*p, *p0;
		int				nb_poss, nb_took;
		int				index;
		BOOL			skip;
		int				curr_coef, best_coef;
		SCORE_TO_SORT	scores_to_sort[ 33 ], *score_to_sort;
		void			*dirs[ 33 ], **pdir;

	/*--- --------------------- ---*/
	/*--- Terminal Evaluation ? ---*/
	/*--- --------------------- ---*/
	
	if ( depth == max_depth )
		{
		cnt++;
		if ( f_refresh_time && (cnt%10000 == 1) )
			do_refresh_time();
		return (*pf_ponderation)( the_player, the_enemy );
		}	// if


	/*--- ---------------------------------------------------------- ---*/
	/*--- Gener all the possibilities of game for the current player ---*/
	/*--- ---------------------------------------------------------- ---*/
	
	ptr1 = ptr2 = curr_ptr;
	nb_poss = do_build_possib( &ptr2, the_player, the_enemy );


	/*--- ---------------- ---*/
	/*--- No possibility ? ---*/
	/*--- ---------------- ---*/
	
	if ( !nb_poss )
		{

		/*--- The two players passes, the game is over ---*/
		if ( prev_pass )
			{
			if ( nb_pawns[ CODE_BLACK ] == nb_pawns[ CODE_WHITE ] )
				return 0;
			else
				if ( nb_pawns[ CODE_BLACK ] > nb_pawns[ CODE_WHITE ] )
					return INT_MAX;
				else 
					return INT_MIN;
			}	// if

		/*--- Game is not over, gnark gnark... ---*/
		return minimax( 
			depth+1, ptr2, TRUE,
			black_best_coef, white_best_coef,
			the_enemy, the_player );
		}	// if


	/*--- ------------------------------------------------------ ---*/
	/*--- Evaluate each move, in order to sort the list of moves ---*/
	/*--- ------------------------------------------------------ ---*/

	if ( depth < (max_depth - 1) )	// Dont't need to sort the last move before ponderation
		for ( score_to_sort = scores_to_sort, index = 0, p = (UINT*)ptr1, p++; 
			  p != ptr2; 
			  index++, score_to_sort++ )
			{

			/*--- Play this position ---*/
			p0 = p++;
			PLAY_CELL( p0, the_player );
			nb_pawns[ the_player ] += ((nb_took = *p++) + 1);
			nb_pawns[ the_enemy ]  -= nb_took;

			/*--- Give a 'note' to the move ---*/
			score_to_sort->index = index;
			score_to_sort->score = (pf_ponderation)( the_player, the_enemy );

			/*--- Unplay this position ---*/
			UNPLAY_CELL( p0, the_enemy );
			nb_pawns[ the_player ] -= (nb_took + 1);
			nb_pawns[ the_enemy ]  += nb_took;

			/*--- Next possibility for this player ---*/
			p += nb_took;

			}	// for (p)


	/*--- ---------------------------- ---*/
	/*--- Then sort this list of moves ---*/
	/*--- ---------------------------- ---*/

	if ( depth < (max_depth - 1) )
		{
	
		/*--- 1a) Sort the list of possibles moves by their coef ---*/
		if ( the_player == CODE_BLACK )
			qsort( scores_to_sort, nb_poss, sizeof( SCORE_TO_SORT ), sort_scores_black );
		else
			qsort( scores_to_sort, nb_poss, sizeof( SCORE_TO_SORT ), sort_scores_white );

		}	// if
	else
		{

		/*--- 1b) Create the list, not ordered ---*/
		for ( score_to_sort = scores_to_sort, index = 0, p = (UINT*)ptr1, p++; 
			  p != ptr2; 
			  index++, score_to_sort++ )
			{
			p++;
			nb_took = *p++;
			score_to_sort->index = index;
			p += nb_took;
			}	// for (p)

		}	// else

	
	/*--- 2) Create a 'directory' of each possible move ---*/
	for ( pdir = dirs, p = (UINT*)ptr1, p++, index = 0; p != ptr2; index++ )
		{
		dirs[ index ] = p++;
		nb_took = *p++;
		p += nb_took;
		}

			
	/*--- --------------------------------- ---*/
	/*--- Initialize alpha-beta coeficients ---*/
	/*--- --------------------------------- ---*/
	
	if ( the_player == CODE_BLACK )	
		best_coef = INT_MIN;		// Black maximize coefs.
	else
		best_coef = INT_MAX;		// White minimize coefs.


	/*--- ------------------------------------- ---*/ 
	/*--- For each (sorted) possibility of play ---*/
	/*--- ------------------------------------- ---*/ 

	for ( skip = FALSE, score_to_sort = scores_to_sort, index = nb_poss; 
		  (index) && (!skip);
		  index--, score_to_sort++ )
		{

		/*--- Play this position ---*/
		p = dirs[ score_to_sort->index ];
		p0 = p++;
		PLAY_CELL( p0, the_player );
		nb_pawns[ the_player ] += ((nb_took = *p++) + 1);
		nb_pawns[ the_enemy ]  -= nb_took;

		curr_coef = minimax( 
			depth+1, ptr2, FALSE,
			black_best_coef, white_best_coef,
			the_enemy, the_player );

		/*--- Mini-Max Algorithm ---*/
		if ( the_player == CODE_BLACK )
			{
			if ( curr_coef > best_coef )
				{
				if ( !depth )
					chosen_move  = *p0;
				if ( (best_coef = curr_coef) > black_best_coef )
					if ( best_coef >= white_best_coef )			// Alpha-Beta Algorithm
						skip = TRUE;
					else
						black_best_coef = best_coef;
				}
			}
		else
			{
			if ( curr_coef < best_coef )
				{
				if ( !depth )
					chosen_move  = *p0;
				if ( (best_coef = curr_coef) < white_best_coef )
					if ( best_coef <= black_best_coef )			// Alpha-Beta Algorithm
						skip = TRUE;
					else
						white_best_coef = best_coef;
				}
			}

		/*--- Unplay this position ---*/
		UNPLAY_CELL( p0, the_enemy );
		nb_pawns[ the_player ] -= (nb_took + 1);
		nb_pawns[ the_enemy ]  += nb_took;

		}	// for (p)

	/*--- Best coef. for this node is ... ---*/
	return best_coef;

	}	// minimax


static void _do_game( const UBYTE the_player, const UBYTE the_enemy,
					  WHERE_TO_PLAY *where_to_play )
	{
	time_t		t1, t2;

	/*--- Initial time ---*/
	time( &t1 );

	/*--- Refresh the time is player is the computer ---*/
	f_refresh_time = (the_player == code_computer) ? TRUE : FALSE;

	cnt = 0;
	minimax(
		0, &possib[ 0 ], FALSE,
		INT_MIN, INT_MAX,
		the_player, the_enemy );

	/*--- Fill the where-to-play structure ---*/
	time( &t2 );
	where_to_play->elapsed = (time_t) difftime( t2, t1 );
	where_to_play->x = ((chosen_move -(UINT)board) / 10);
	where_to_play->y = ((chosen_move -(UINT)board) % 10);
	where_to_play->cnt = cnt;

	}	// _do_game_...
