/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "phothello.h"
#include "abimport.h"
#include "proto.h"


int
gridcell_activate( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
	{
		PhPointerEvent_t	*infos = (PhPointerEvent_t *) cbinfo->event;
		int					x, y;
		BOOL				found;
		int					wgt_name;
		void				*ptr1, *ptr2;
		UINT				*p;
		void				*possib[ 256 ];
		int					nb_poss, nb_took;
		int					k;
		BOOL				valid_move;
		int					hist_ptr;
		time_t				t_end;
static	int					computer = FALSE;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	infos = (PhPointerEvent_t *)PhGetData( cbinfo->event );
	if ( cbinfo->event->type == Ph_EV_BUT_RELEASE )
		{
		if ( (cbinfo->event->subtype == Ph_EV_RELEASE_ENDCLICK) &&
			 (infos->click_count >= 2) )
			{
			if ( computer )
				{
				computer = FALSE;
				PtFlush();
				phothello_btn_computer_turn( ABW_phothello_btn_computer_turn, NULL, cbinfo );
				PtFlush();
				}
			}
		return( Pt_CONTINUE );
		}

	/*--- Take not allowed ? ---*/
	if ( which_show_poss != code_human )
		return( Pt_CONTINUE );

	/*--- Is-it a cell ? Or a pawn ? ---*/
	if ( ((wgt_name = ApName( widget )) >= ABN_phothello_pos_A1) &&
		(wgt_name <= ABN_phothello_pos_G8) )
		{
		for ( found = FALSE, x = 0; (x < 8) && !found; x++ )
			for ( y = 0; (y < 8) && !found; y++ )
				if ( wgt_name == (ABN_phothello_pos_A1 + x*8 + y) )
					found = TRUE;
		}
	else
		{
		for ( found = FALSE, x = 0; (x < 8) && !found; x++ )
			for ( y = 0; (y < 8) && !found; y++ )
				if ( wgt_name == (ABN_pawn_A1 + x*8 + y) )
					found = TRUE;
		}
	if ( !found )
		return( Pt_CONTINUE );

	/*--- Show the enclosed enemy pawns ---*/
	ptr1 = ptr2 = &possib[ 0 ];
	if ( !(nb_poss = do_build_possib( &ptr2, code_human, code_computer )) )
		return( Pt_CONTINUE );
	for ( valid_move = FALSE, p = (UINT*)ptr1, p++; p != ptr2; )
		{
		if ( (UBYTE*)*p++ == &board[ XY(x, y)] )
			{
			board[ XY(x, y) ] = code_human;
			nb_took = *p++;
			memmove( cancel_last_human_move, &p[ -2 ], (nb_took+2) * sizeof( void* ) );
			for ( k = 0; k < nb_took; k++ )
				*(UBYTE*)*p++ = code_human;
			valid_move = TRUE;
			break;
			}
		nb_took = *p++;
		p += nb_took;
		}	// for (p)

	/*--- Update the board ---*/
	if ( !valid_move )
		return( Pt_CONTINUE );

	/*--- Hide the button 'This is your turn' ---*/
	PtUnrealizeWidget( ABW_phothello_btn_human_turn );

	/*--- Show the modified board ---*/
	draw_complete_board();
	compute_nb_pawns( TRUE );

	/*--- Update the list of move ---*/
	hist_ptr = getItemSelectedList( ABW_phothello_history ) + 1;
	addListItem( ABW_phothello_history, "%2d: %s : %c%d",
		hist_ptr,
		lib_colors[ code_human ],
		y + '@', x );
	PtListDeleteItemPos( ABW_phothello_history, 60, hist_ptr+1 );
	selItemSelectedList( ABW_phothello_history, getItemsCountList( ABW_phothello_history ) );

	/*--- Message: where human played ---*/
	setString( ABW_phothello_msg, "Your play was %c%d",
		y + '@', x );

	/*--- Update the time took by the player ---*/
	time( &t_end );
	if ( code_human == CODE_BLACK )
		{
		ctime_black += ((unsigned long)t_end - (unsigned long)t_start);
		setString( ABW_phothello_ctime_black, "%02d:%02d",
			ctime_black / 60, ctime_black % 60 );
		}
	else
		{
		ctime_white += ((unsigned long)t_end - (unsigned long)t_start);
		setString( ABW_phothello_ctime_white, "%02d:%02d",
			ctime_white / 60, ctime_white % 60 );
		}

	/*--- Show the possibilities for the computer ---*/
	if ( (nb_poss = show_poss_play_cursor( code_computer, code_human )) == 0 )
		{
		ptr2 = &possib[ 0 ];
		if ( do_build_possib( &ptr2, code_human, code_computer ) == 0 )
			return diag_end_game();
		}
	PtRealizeWidget( ABW_phothello_btn_human_cancel );

	/*--- Selection of the last element of the history list ---*/
	selItemSelectedList( ABW_phothello_history, getItemsCountList( ABW_phothello_history ) );

	/*--- How many possibilities for the computer ? ---*/
	if ( nb_poss == 0 )
		{
		computer = FALSE;
		PtRealizeWidget( ABW_phothello_btn_computer_pass );
		PtContainerGiveFocus( ABW_phothello_btn_computer_pass, cbinfo->event );
		color_to_play = code_human;
		}
	else
		{
		computer = TRUE;
		PtRealizeWidget( ABW_phothello_btn_computer_turn );
		PtContainerGiveFocus( ABW_phothello_btn_computer_turn, cbinfo->event );
		color_to_play = code_computer;
		}

	/*--- Enable the menu option 'Save the game' ---*/
	ApModifyItemState( &file_menu, AB_ITEM_NORMAL, 
		ABN_file_save_game,
		NULL );

	/*--- Stop the clock for the player ---*/
	stop_count_time();

	return( Pt_CONTINUE );

	}	// gridcell_activate


int
gridcell_boundary( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	int			x, y;
	BOOL		found;
	int			wgt_name;
	void		*ptr1, *ptr2;
	UINT		*p;
	void		*possib[ 256 ];
	int			nb_poss, nb_took;
	int			k;
	BOOL		valid_move;
	int			the_player, the_enemy;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Take nor allowed ? ---*/
	if ( which_show_poss == CODE_EMPTY )
		return( Pt_CONTINUE );
	the_player = which_show_poss;
	the_enemy  = (CODE_BLACK + CODE_WHITE) - the_player;

	/*--- Nothing to do ? ---*/
	if ( cbinfo->event->type != Ph_EV_BOUNDARY )
		return( Pt_CONTINUE );

	/*--- Is-it a cell ? Or a pawn ? ---*/
	if ( ((wgt_name = ApName( widget )) >= ABN_phothello_pos_A1) &&
		(wgt_name <= ABN_phothello_pos_G8) )
		{
		for ( found = FALSE, x = 0; (x < 8) && !found; x++ )
			for ( y = 0; (y < 8) && !found; y++ )
				if ( wgt_name == (ABN_phothello_pos_A1 + x*8 + y) )
					found = TRUE;
		}
	else
		{
		for ( found = FALSE, x = 0; (x < 8) && !found; x++ )
			for ( y = 0; (y < 8) && !found; y++ )
				if ( wgt_name == (ABN_pawn_A1 + x*8 + y) )
					found = TRUE;
		}
	if ( !found )
		return( Pt_CONTINUE );

	/*--- Cell not empty ---*/
	if ( board[ XY( x, y ) ] != CODE_EMPTY )
		return( Pt_CONTINUE );


	/*--- ------------------------------- ---*/
	/*--- Enter or Leave this grid cell ? ---*/
	/*--- ------------------------------- ---*/
	
	switch ( cbinfo->event->subtype )
		{
		
		case Ph_EV_PTR_ENTER :

			/*--- Show the enclosed enemy pawns ---*/
			ptr1 = ptr2 = &possib[ 0 ];
			nb_poss = do_build_possib( &ptr2, the_player, the_enemy );
			if ( nb_poss == 0 )
				return( Pt_CONTINUE );
	
			/*--- Change the board with the new pawns and the enclosed pawns ---*/
			for ( valid_move = FALSE, p = (UINT*)ptr1, p++; p != ptr2; )
				{
				if ( (UBYTE*)*p++ == &board[ XY(x, y)] )
					{
					board[ XY(x, y) ] = the_player;
					for ( nb_took = *p++, k = 0; k < nb_took; k++ )
						*(UBYTE*)*p++ = the_player;
					valid_move = TRUE;
					break;
					}
				nb_took = *p++;
				p += nb_took;
				}	// for (p)
			if ( !valid_move )
				return( Pt_CONTINUE );
			draw_complete_board();
			compute_nb_pawns( TRUE );

			/*--- Redraw the original board ---*/
			for ( p = (UINT*)ptr1, p++; p != ptr2; )
				{
				if ( (UBYTE*)*p++ == &board[ XY(x, y)] )
					{
					board[ XY(x, y) ] = CODE_EMPTY;
					for ( nb_took = *p++, k = 0; k < nb_took; k++ )
						*(UBYTE*)*p++ = the_enemy;
					break;
					}
				nb_took = *p++;
				p += nb_took;
				}	// for (p)

			break;

		case Ph_EV_PTR_LEAVE :

			/*--- Show the enclosed enemy pawns ---*/
			draw_complete_board();
			compute_nb_pawns( TRUE );
			
			break;

		}	// switch

	return( Pt_CONTINUE );

	}	// gridcell_boundary
