/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.12E */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "phothello.h"
#include "abimport.h"
#include "proto.h"


int
phothello_setup( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Initializations prior to begin a new game ---*/
	init_new_game( TRUE, TRUE );

	/*--- Set the Position/Size of the zoom window if given in the .ini file ---*/
	readandset_cfg_szpos( "Phothello window", ABW_phothello );

	/*--- First disable some options of the menu ---*/
	ApModifyItemState( &file_menu, AB_ITEM_DIM, 
		ABN_file_save_game,
		NULL );

	return( Pt_CONTINUE );

	}	// phothello_setup


int
phothello_realized( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Get options. If not exist, show help window ---*/
	if ( read_cfg_options() == Pt_FALSE )
		ApCreateModule( ABM_rules, NULL, NULL );

	return( Pt_CONTINUE );

	}	// phothello_realized


void computer_play( WHERE_TO_PLAY *where_to_play,
					const UBYTE the_player, const UBYTE the_enemy,
					const BOOL f_suggest )
	{
	int			total_nb_pawns;
	int			nb_eog;
	char		msg[ 80 ];	

	/*--- Hourglass ON ---*/
	busyWindow( ABW_phothello, TRUE );
	
	/*--- Thinking... ---*/
	setString( ABW_phothello_msg, "Thinking..." );
	PtFlush();

	/*--- How many pawns remainding ? ---*/
	total_nb_pawns = nb_pawns[ CODE_BLACK ] + nb_pawns[ CODE_WHITE ];

	/*--- End of game ---*/
	switch ( options.level )
		{
		case LEVEL1	: nb_eog = 64 -  6; break;
		case LEVEL2	: nb_eog = 64 -  8; break;
		case LEVEL3	: nb_eog = 64 - 10; break;
		case LEVEL4	: nb_eog = 64 - 12; break;
		}	// switch

	/*--- Call the routine to compute where to play ---*/
	if ( total_nb_pawns >= nb_eog )
		do_end_game( the_player, the_enemy, where_to_play );
	else
		do_game( the_player, the_enemy, where_to_play );

	/*--- Hourglass OFF ---*/
	busyWindow( ABW_phothello, FALSE );
	
	/*--- Show where to play the 'best' position ---*/
	if ( where_to_play->x == -1 )
		setString( ABW_phothello_msg, "No possibility.\nMust pass..." );
	else
		{
		sprintf( msg, "%s configurations examined in %d sec",
			str_nb( where_to_play->cnt ), 
			where_to_play->elapsed );
		if ( !f_suggest )
			setString( ABW_phothello_msg, "%s\nComputer move is %c%d",
				msg, where_to_play->y + '@', where_to_play->x );
		else
			if ( the_player == code_computer )
				setString( ABW_phothello_msg, "%s\nComputer move would be %c%d",
					msg, where_to_play->y + '@', where_to_play->x );
			else
				setString( ABW_phothello_msg, "%s\nYour 'best' move would be %c%d",
					msg, where_to_play->y + '@', where_to_play->x );
		}

	}	// computer_play


int
phothello_btn_computer_turn( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	WHERE_TO_PLAY	where_to_play;
	void			*ptr1, *ptr2;
	UINT			*p;
	void			*possib[ 256 ];
	int				nb_poss, nb_took;
	int				k;
	BOOL			valid_move;
	int				hist_ptr;
	time_t			t_end;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- No game possibility for the player ---*/
	noshow_poss_play_cursor();

	/*--- Hide the button: "Computer's turn: ---*/
	PtUnrealizeWidget( ABW_phothello_btn_computer_turn );
	PtUnrealizeWidget( ABW_phothello_btn_human_cancel );

	/*--- Start the clock for the current player ---*/
	start_count_time( code_computer );

	/*--- The computer plays... ---*/
	time( &t_start );
	computer_play( &where_to_play, code_computer, code_human, FALSE );
	time( &t_end );
	if ( code_computer == CODE_BLACK )
		{
		ctime_black += ((unsigned long)t_end - (unsigned long)t_start);
		setString( ABW_phothello_ctime_black, "%02d:%02d",
			ctime_black / 60, ctime_black % 60 );
		}
	else
		{
		ctime_white += ((unsigned long)t_end - (unsigned long)t_start);
		setString( ABW_phothello_ctime_white, "%02d:%02d",
			ctime_white / 60, ctime_white % 60 );
		}
	time( &t_start );				// Time when player starts thinking

	/*--- Show the enclosed enemy pawns ---*/
	ptr1 = ptr2 = &possib[ 0 ];
	if ( (nb_poss = do_build_possib( &ptr2, code_computer, code_human )) == 0 )
		return( Pt_CONTINUE );
	for ( valid_move = FALSE, p = (UINT*)ptr1, p++; p != ptr2; )
		{
		if ( (UBYTE*)*p++ == &board[ XY(where_to_play.x, where_to_play.y)] )
			{
			board[ XY(where_to_play.x, where_to_play.y) ] = code_computer;
			for ( nb_took = *p++, k = 0; k < nb_took; k++ )
				*(UBYTE*)*p++ = code_computer;
			valid_move = TRUE;
			break;
			}
		for ( nb_took = *p++, k = 0; k < nb_took; k++ )
			p++;
		}	// for (p)

	/*--- Update the board ---*/
	if ( !valid_move )
		return( Pt_CONTINUE );

	/*--- Redraw the board ---*/
	draw_complete_board();
	compute_nb_pawns( TRUE );

	/*--- Update the list of move (history) ---*/
	hist_ptr = getItemSelectedList( ABW_phothello_history ) + 1;
	addListItem( ABW_phothello_history, "%2d: %s : %c%d",
		hist_ptr,
		lib_colors[ code_computer ],
		where_to_play.y + '@', where_to_play.x );
	PtListDeleteItemPos( ABW_phothello_history, 60, hist_ptr+1 );
	selItemSelectedList( ABW_phothello_history, getItemsCountList( ABW_phothello_history ) );
			
	/*--- End-of-Game ? ---*/
	if ( (nb_poss = show_poss_play_cursor( code_human, code_computer )) == 0 )
		{
		ptr2 = &possib[ 0 ];
		if ( do_build_possib( &ptr2, code_computer, code_human ) == 0 )
			return diag_end_game();
		}

	/*--- Update the cursors into the grid, depending the game possibilities ---*/
	if ( nb_poss == 0 )
		{
		PtRealizeWidget( ABW_phothello_btn_human_pass );
		PtContainerGiveFocus( ABW_phothello_btn_human_pass, cbinfo->event );
		color_to_play = code_computer;
		}
	else
		{
		PtRealizeWidget( ABW_phothello_btn_human_turn );
		PtContainerGiveFocus( ABW_phothello_btn_human_turn, cbinfo->event );
		color_to_play = code_human;
		}
		
	/*--- Selection of the last element of the history list ---*/
	selItemSelectedList( ABW_phothello_history, getItemsCountList( ABW_phothello_history ) );

	/*--- Enable the menu option 'Save the game' ---*/
	ApModifyItemState( &file_menu, AB_ITEM_NORMAL, 
		ABN_file_save_game,
		NULL );

	/*--- Start the clock for the player now ---*/
	stop_count_time();
	start_count_time( code_human );

	return( Pt_CONTINUE );

	}	// phothello_btn_computer_turn


int
phothello_btn_human_pass( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Hide the button: "You must pass" ---*/
	PtUnrealizeWidget( ABW_phothello_btn_human_pass );

	/*--- Show the button: "Computer's turn: ---*/
	PtRealizeWidget( ABW_phothello_btn_computer_turn );

	/*--- Just a message ---*/
	setString( ABW_phothello_msg, "You have passed your turn..." );

	/*--- No game possibility for the player ---*/
	noshow_poss_play_cursor();

	return( Pt_CONTINUE );

	}	// phothello_btn_human_pass


int
phothello_btn_human_cancel( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{
	UINT		*p;
	int			xy, x, y;
	int			nb_took;
	int			k;
	int			hist_sz;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Hide some buttons ---*/
	PtUnrealizeWidget( ABW_phothello_btn_human_cancel );
	PtUnrealizeWidget( ABW_phothello_btn_computer_turn );
	PtUnrealizeWidget( ABW_phothello_btn_computer_pass );

	/*--- Cancel the last move ---*/
	p = (UINT*)cancel_last_human_move;
	xy = (UINT)*p++ - (UINT)board;
	y = xy % 10;
	x = xy / 10;
	board[ XY(x, y) ] = CODE_EMPTY;
	for ( nb_took = *p++, k = 0; k < nb_took; k++ )
		*(UBYTE*)*p++ = code_computer;

	/*--- Show the modified board ---*/
	draw_complete_board();
	compute_nb_pawns( TRUE );

	/*--- Update the list of move (history) ---*/
	hist_sz = getItemsCountList( ABW_phothello_history );
	PtListDeleteItemPos( ABW_phothello_history, 60, hist_sz );
	selItemSelectedList( ABW_phothello_history, getItemsCountList( ABW_phothello_history ) );
			
	/*--- Update the cursors into the grid, depending the game possibilities ---*/
	if ( show_poss_play_cursor( code_human, code_computer ) == 0 )
		{
		PtRealizeWidget( ABW_phothello_btn_human_pass );
		PtContainerGiveFocus( ABW_phothello_btn_human_pass, cbinfo->event );
		}
	else
		{
		PtRealizeWidget( ABW_phothello_btn_human_turn );
		PtContainerGiveFocus( ABW_phothello_btn_human_turn, cbinfo->event );
		}
		
	/*--- Next color to play ---*/
	color_to_play = code_human;
	
	return( Pt_CONTINUE );

	}	// phothello_btn_human_cancel


int
phothello_btn_computer_pass( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Hide the button 'Computer pass' ---*/
	PtUnrealizeWidget( ABW_phothello_btn_computer_pass );
	PtUnrealizeWidget( ABW_phothello_btn_human_cancel );

	/*--- Update the cursors into the grid, depending the game possibilities ---*/
	if ( show_poss_play_cursor( code_human, code_computer ) == 0 )
		return diag_end_game();
	
	/*--- Human's turn ---*/
	PtRealizeWidget( ABW_phothello_btn_human_turn );
	PtContainerGiveFocus( ABW_phothello_btn_human_turn, cbinfo->event );
		
	return( Pt_CONTINUE );

	}	// phothello_btn_computer_pass


int
phothello_menubar_realized( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	/*--- Hide some buttons ---*/
	PtUnrealizeWidget( ABW_phothello_btn_computer_turn );
	PtUnrealizeWidget( ABW_phothello_btn_computer_pass );
	PtUnrealizeWidget( ABW_phothello_btn_human_turn );
	PtUnrealizeWidget( ABW_phothello_btn_human_pass );
	PtUnrealizeWidget( ABW_phothello_btn_human_cancel );

	/*--- Hide the clock ---*/
	PtUnrealizeWidget( ABW_phothello_rtime_black );
	PtUnrealizeWidget( ABW_phothello_rtime_white );

	return( Pt_CONTINUE );

	}	// phothello_menubar_realized


int diag_end_game( void )
	{

#if LOGTOTMP==1
	slog( "computer:%d  %s:%d  [level=%d, strategy=%d]", 
		nb_pawns[ code_computer ],
		getlogin(), nb_pawns[ code_human ],
		options.level, options.strategy );
#endif

	if ( nb_pawns[ code_human ] > nb_pawns[ code_computer ] )
		setString( ABW_phothello_msg, "CONGRATULATIONS !\nYou win against Computer !" );
	else if ( nb_pawns[ code_human ] < nb_pawns[ code_computer ] )
		setString( ABW_phothello_msg, "SORRY !\nYou loose against Computer..." );
	else
		setString( ABW_phothello_msg, "This is a draw game !" );

	return( Pt_CONTINUE );

	}	// diag_end_game


int
phothello_destroyed( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

#if LOGTOTMP==1
	slog( "exit" );
#endif

	/*--- Upcate the .ini file (configuration file) ---*/
	update_config_file();

	return( Pt_CONTINUE );

	}	// phothello_destroyed
