#ifndef COMMON
	#define COMMON extern
#endif

// Version number of software
#define VERSION_NB		0x010001		// version 1.0.B
#define VERSION_DATE	0x000806		// yymmdd

// Options
#define LOGTOTMP		1

/*--- Common things ---*/
typedef unsigned int UINT;
typedef short int SHORT;
typedef short unsigned int USHORT;
typedef signed char BYTE;
typedef unsigned char UBYTE;
typedef signed char BOOL;
#define FALSE 0
#define TRUE -1

/*--- Configuration file ---*/
#define CFG_FILENAME			"phothello.ini"
#define CFG_SECTION_WINSZPOS	"Windows Size and Position"
#define	CFG_SECTION_OPTIONS		"Options"

/*--- Board pawns codes ---*/
#define	CODE_AROUND			-1
#define	CODE_EMPTY			0
#define	CODE_BLACK			1
#define	CODE_WHITE			2
#ifdef __MAIN__
char	*lib_colors[ 3 ] =
	{ "empty", "black", "white" };
#else
extern	char	*lib_colors[ 3 ];
#endif

/*--- Convert a board coordonnate into a linar position ---*/
#define XY( x, y )	((x)*10+(y))

/*--- Show a move name in a form A1-H8 ---*/
#define SHOW_COORD_Y( p )	((((UINT)p - (UINT)board) % 10) + '@')
#define SHOW_COORD_X( p )	(((UINT)p - (UINT)board) / 10)

/*--- The game ---*/
COMMON	UBYTE	code_computer;				// Either CODE_BLACK or CODE_WHITE
COMMON	UBYTE	code_human;					// Either CODE_BLACK or CODE_WHITE
COMMON	UBYTE	color_to_play;				// Next color to play
COMMON	UBYTE	board[ (8+2) * (8+2) ];
COMMON	int		nb_pawns[ 3 ];				// CODE_BLACK+1 = 3
COMMON	void	*cancel_last_human_move[ 40 ];	// To cancel the last human move
COMMON	int		ctime_black, ctime_white;	// Cumul time played by each color
COMMON	time_t	t_start;					// Time when player starts thinking

/*--- Couple Proxy/Timer signaled each second to refresh the time ---*/
COMMON	time_t	start_refreshTime;
COMMON	UBYTE	color_refreshTime;
COMMON	pid_t	proxy_refreshTime;
COMMON	timer_t	timer_refreshTime;

/*--- List (stack) of possibilities *and* enclosed pawns ---*/
COMMON	void	*possib[ 4096 ];
#pragma aux do_build_possib parm [edi][eax][ebx] value [eax];
extern int do_build_possib( const void **ptr, const int cplayer, const int cenemy );

/*--- Compute the number of possibilities ---*/
#pragma aux nb_possib parm [eax][ebx] value [eax];
extern int nb_possib( const int cplayer, const int cenemy );

/*--- If this variable is TRUE, show the possibilities of the user ---*/
/*--- Meaning:	0 means no show, else the color to show ---*/
COMMON	int		which_show_poss;

/*--- Code the configuration of the edges ---*/
extern int code_edges( void );
COMMON	UINT	edge_north, edge_east, edge_south, edge_west;

/*--- Current ponderation function to execute ---*/
COMMON int(*pf_ponderation)( const int the_player, const int the_enemy );

/*--- Options (settings) ---*/
enum { LEVEL1=1, LEVEL2, LEVEL3, LEVEL4, MAX_LEVELS };
enum { STRATEGY1=1, STRATEGY2, STRATEGY3, STRATEGY4, STRATEGY5, STRATEGY6, MAX_STRATEGIES };
typedef struct
	{
	int		level;						// Level of computer's level
	int		strategy;					// Which stategy ?
	}	OPTIONS;
COMMON	OPTIONS	options;
#ifdef __MAIN__
	UBYTE	levels[ MAX_LEVELS-1 ][ MAX_STRATEGIES-1 ] = 
		{
		{  4,  4,  4,  4,  4,  4 },			// Level #1
		{  6,  5,  5,  6,  6,  6 },			// Level #2
		{  8,  8,  8,  7,  8,  7 },			// Level #3
		{ 10, 10, 10, 10, 10, 10 }			// Level #4
		};
#else
	extern UBYTE	levels[ MAX_LEVELS-1 ][ MAX_STRATEGIES-1 ];
#endif

/*--- Position to play ---*/
typedef struct
	{
	int		x, y;						// Position to play, set to (-1, -1) if pass
	int		coef_best;					// Best coef. 
	long	cnt;						// Nb of positions examined
	int		max_level;					// Max level reached
	time_t	elapsed;					// Time set for calculation
	}	WHERE_TO_PLAY;

/*--- Used to sort scores into the minimax function ---*/
typedef struct
	{
	int		score;
	int		index;
	}	SCORE_TO_SORT;

/*--- .ini filename ---*/
COMMON	char	ini_fname[ PATH_MAX+30];
