/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1. 0  */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <sys/proxy.h>
#include <sys/kernel.h>
#include <pwd.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#define COMMON
#define	__MAIN__
#include "phothello.h"

#include "abimport.h"
#include "proto.h"


void do_refresh_time( void )
	{
	time_t		now;
	int			delta;
	
	/*--- Show the current time ---*/
	time( &now );
	delta = (unsigned long)now - (unsigned long)start_refreshTime;
	if ( color_refreshTime == CODE_BLACK )
		setString( ABW_phothello_rtime_black, "%02d:%02d", delta/60, delta%60 );
	else
		setString( ABW_phothello_rtime_white, "%02d:%02d", delta/60, delta%60 );
	PtFlush();

	}	// do_refresh_time


// ----------------------------------------------------------------------------
// This function is called when the proxy proxyRefreshPcTime is signaled,
// i.e. each second: the purpose is to refresh on main screen the current
// date./time for each connected SrX
// ----------------------------------------------------------------------------
int refreshTime_handler( void *data, pid_t pid, void *msg, ushort_t size )
	{	

	/* eliminate 'unreferenced' warnings */
	data = data, pid = pid, msg = msg, size = size;	

	/*--- Show the current time ---*/
	do_refresh_time();

	/*--- Flush still pending proxy ---*/
	while ( Creceive( proxy_refreshTime, NULL, 0 ) == proxy_refreshTime );

	return Pt_CONTINUE;
	
	}	// refreshTime_handler


void start_count_time( const UBYTE color )
	{
	struct itimerspec	timerSpec;

	/*--- Initialise the starting time (reference) ---*/
	time( &start_refreshTime );
	color_refreshTime = color;

	/*--- Fired each second ---*/
	timerSpec.it_value.tv_sec     = 1;		// 1 sec. before initial firing
	timerSpec.it_value.tv_nsec    = 0;
	timerSpec.it_interval.tv_sec  = 1;		// repetitive timer afterwards
	timerSpec.it_interval.tv_nsec = 0;
	timer_settime( timer_refreshTime, 0, &timerSpec, NULL );

	/*--- Show initialisation ---*/
	if ( color_refreshTime == CODE_BLACK )
		{
		setString( ABW_phothello_rtime_black, "00:00" );
		PtRealizeWidget( ABW_phothello_rtime_black );
		}
	else
		{
		setString( ABW_phothello_rtime_white, "00:00" );
		PtRealizeWidget( ABW_phothello_rtime_white );
		}

	}	// start_count_time


void stop_count_time( void )
	{
	struct itimerspec	timerSpec;

	/*--- Disable the timer ---*/
	timerSpec.it_value.tv_sec     = 0;		// 0 means: disable
	timerSpec.it_value.tv_nsec    = 0;
	timer_settime( timer_refreshTime, 0, &timerSpec, NULL );

	/*--- Hide the clock ---*/
	if ( color_refreshTime == CODE_BLACK )
		PtUnrealizeWidget( ABW_phothello_rtime_black );
	else
		PtUnrealizeWidget( ABW_phothello_rtime_white );

	}	// stop_count_time


#if LOGTOTMP==1
void slog( const char *fmt, ... )
	{
	va_list			ap;
	char			msg[ 300 ];
	FILE			*fp;
	time_t			now;
	struct tm		*tm;

	/*--- Develop text to log ---*/
	va_start( ap, fmt );
	vsprintf( msg, fmt, ap );
	va_end( ap );

	/*--- Update log file ---*/
	time( &now );
	tm = localtime( &now );
	if ( (fp = fopen( "/tmp/log.txt", "a" )) != NULL )
		{
		fprintf( fp, "%02d/%02d %02d:%02d:%02d [phothello] [%s] %s\n",
			tm->tm_mday, tm->tm_mon+1,
			tm->tm_hour, tm->tm_min, tm->tm_sec,
			getlogin(), 
			msg );
		fclose( fp );
		}

	}	// slog
#endif


int
phothello_init( int argc, char *argv[] )
	{
	struct sigevent		sigevnt;
	struct passwd		*pw;

	/* eliminate 'unreferenced' warnings */
	argc = argc, argv = argv;

#if LOGTOTMP==1
	slog( "start" );
#endif

	/*--- 16K buffer for Photon ---*/
	PgSetDrawBufferSize( 4*1024 );

	/*--- Dialogs not yet mounted ---*/
	ABW_settings =
	ABW_rules    = 
	ABW_about	 = NULL;

	/*--- .ini filename ---*/
	strcpy( ini_fname, "" );
	if ( getlogin() && ((pw = getpwnam( getlogin() )) != NULL) )
		{
		char	p[ PATH_MAX+1 ];
		int		l;
		strcpy( p, pw->pw_dir );
		if ( ((l=strlen(p)) >= 1) && (p[l-1]=='/') )
			p[l-1] = 0;
		sprintf( ini_fname, "%s/.photon/%s", p, CFG_FILENAME );
		}

	/*--- Open the main window ---*/
	ApCreateModule( ABM_phothello, NULL, NULL );

	/*--- Function signaled each second to refresh PC date/time ---*/
	if ( (proxy_refreshTime = qnx_proxy_attach( 0, 0, 0, -1 )) == -1 )
		{
		fprintf( stderr, "No more proxy available.\n" );
		exit( -1 );
		}
	sigevnt.sigev_signo = -proxy_refreshTime;
	if ( (timer_refreshTime = timer_create( CLOCK_REALTIME, &sigevnt )) == -1 ) 
		{
		fprintf( stderr, "No more timer available.\n" );
		exit( -1 );
		}
	PtAppAddInput( NULL, proxy_refreshTime, refreshTime_handler, NULL );

	/*--- Start the clock for the player now ---*/
	start_count_time( code_human );

	/*--- 'This is your turn' ---*/
	PtRealizeWidget( ABW_phothello_btn_human_turn );
	PtContainerGiveFocus( ABW_phothello_btn_human_turn, NULL );

	return( Pt_CONTINUE );

	}	// phothello_init
