/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "phothello.h"
#include "abimport.h"
#include "proto.h"


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static void setCursor_grid( const int x, const int y, const BOOL enable )
	{

	setCursorType( dsphothello[ ABN_phothello_pos_A1 + (x-1)*8 + (y-1) ].wgt,
		(enable) ? Ph_CURSOR_CROSSHAIR : Ph_CURSOR_NOINPUT );
	setCursorType( dsphothello[ ABN_pawn_A1 + (x-1)*8 + (y-1) ].wgt,
		(enable) ? Ph_CURSOR_CROSSHAIR : Ph_CURSOR_NOINPUT );
	
	}	// setCursor_grid


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static void setSelectable_grid( const int x, const int y, const BOOL enable )
	{
		PtArg_t			args[ 3 ];
static	PtRawCallback_t	cb_raw[]      = {
	{ Ph_EV_BOUNDARY,  gridcell_boundary, NULL },
	{ Ph_EV_BUT_PRESS | Ph_EV_BUT_RELEASE, gridcell_activate, NULL }
	};

	PtSetArg( &args[ 0 ], Pt_ARG_FLAGS, (enable) ? Pt_TRUE : Pt_FALSE, Pt_SELECTABLE );
	PtSetArg( &args[ 1 ], Pt_CB_RAW, cb_raw, sizeof( cb_raw ) / sizeof( cb_raw[ 0 ] ) );
	PtSetResources( dsphothello[ ABN_phothello_pos_A1 + (x-1)*8 + (y-1) ].wgt, 2, args );
	PtSetResources( dsphothello[ ABN_pawn_A1 + (x-1)*8 + (y-1) ].wgt, 2, args );

	}	// setSelectable_grid


void compute_nb_pawns( const BOOL show )
	{
	int		x, y;

	/*--- Compute the nb of pawns ---*/	
	nb_pawns[ CODE_WHITE ] = nb_pawns[ CODE_BLACK ] = 0;
	for ( x = 1; x <= 8; x++ )
		for ( y = 1; y <= 8; y++ )
			nb_pawns[ board[ XY( x, y ) ] ]++;
	
	/*--- Update the board ---*/
	if ( show )
		{
		setString( ABW_phothello_nb_white, "%d", nb_pawns[ CODE_WHITE ] );
		setString( ABW_phothello_nb_black, "%d", nb_pawns[ CODE_BLACK ] );
		}	// if (show)
	
	}	// compute_nb_pawns


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void draw_pawn( const int x, const int y, const int code_pawn )
	{
	PtArg_t		args[ 2 ];
	PtWidget_t	*wgt;
	PgColor_t	fill_color, round_color;

	wgt = dsphothello[ ABN_pawn_A1 + ((x-1)<<3) + (y-1) ].wgt;

	switch ( code_pawn )
		{
		case CODE_EMPTY			: fill_color = round_color = Pg_TRANSPARENT;     break;
		case CODE_BLACK			: fill_color = Pg_BLACK; round_color = Pg_WHITE; break;
		case CODE_WHITE			: fill_color = Pg_WHITE; round_color = Pg_BLACK; break;
		default					: fill_color = round_color = Pg_RED; break;
		}
	PtSetArg( &args[ 0 ], Pt_ARG_FILL_COLOR, fill_color, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_COLOR, round_color, 0 );
	PtSetResources( wgt, 2, args );
	
	}	// draw_pawn


void draw_complete_board( void )
	{
	int		x, y;

	/*--- Draw each pawn ---*/	
	for ( x = 1; x <= 8; x++ )
		for ( y = 1; y <= 8; y++ )
			draw_pawn( x, y, board[ XY( x, y ) ] );

	PtFlush();

	}	// draw_complete_board


// === show_poss_play_cursor =================================================================
//
// Check each cell. 
// If player can play, then change the cursor shape (ok).
// Else change also the cursor shape (nok).
// A local possibilities stack is used.
//
// ===========================================================================================
int show_poss_play_cursor( const UBYTE cplayer, const UBYTE cenemy )
	{
	int		x, y;
	void	*ptr1, *ptr2;
	UINT	*p;
	void	*possib[ 256 ];
	int		nb_poss, nb_took;
	BOOL	state;
	
	ptr1 = ptr2 = &possib[ 0 ];
	nb_poss = do_build_possib( &ptr2, cplayer, cenemy );

	for ( state = FALSE, x = 1; x <= 8; x++ )
		{
		for ( y = 1; y <= 8; y++ )
			{

			if ( nb_poss )
				{
				for ( state = FALSE, p = (UINT*)ptr1, p++; p != ptr2; )
					{
					if ( (UBYTE*)*p++ == &board[ XY(x, y)] )
						{
						state = TRUE;
						break;
						}
					nb_took = *p++;
					p += nb_took;
					}	// for (p)
				}	// if

			setCursor_grid( x, y, state );
			setSelectable_grid( x, y, state );

			}	// for (y)
		}	// for (x)

	/*--- Show the possibilities of the player ---*/
	which_show_poss = cplayer;

	return nb_poss;

	}	// show_poss_play_cursor


void noshow_poss_play_cursor( void )
	{
	int		x, y;

	/*--- Set each cell not selectable ---*/	
	for ( x = 1; x <= 8; x++ )
		{
		for ( y = 1; y <= 8; y++ )
			{
			setCursor_grid( x, y, FALSE );
			setSelectable_grid( x, y, FALSE );
			}	// for (y)
		}	// for (x)

	/*--- No show the possibilities of the user ---*/
	which_show_poss = CODE_EMPTY;

	}	// noshow_poss_play_cursor


char *str_nb( const int nb )
	{
static	char	str[ 80 ];
		int		l, k, w;
		char	*p, *q, tmp[ 80 ];

	sprintf( tmp, "%d", nb );
	if ( (l = strlen( tmp )) <= 3 )
		return strcpy( str, tmp );

	w = l%3;
	for ( p = str, q = tmp, k = 0; k < l; k++ )
		{
		if ( k%3 == w )
			*p++ = ',';
		*p++ = *q++;
		}
	*p = 0;
	return str;

	}	// str_nb
