/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "phothello.h"
#include "abimport.h"
#include "proto.h"


/*--- -----------------------------------------------------------------	---*/
/*--- Function: message											 		---*/
/*--- 															 		---*/
/*--- Usage:	Print a message inside a dialog window. Just ok. 		---*/
/*--- -----------------------------------------------------------------	---*/
int message( const char *title, const char *fmt, ... )
	{
	va_list ap;
	char    buff[ 384 ];

	va_start( ap, fmt );
	vsprintf( buff, fmt, ap );
	va_end( ap );

	PtMessageBox( ABW_phothello, title, buff, "helv14.phf", "  OK  " );

	return -1;

	}	// message


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void setString( const PtWidget_t *wgt, const char *fmt, ... )
	{
	PtArg_t		args[ 1 ];
	va_list		ap;
	char		buff[ 200 ];

	va_start( ap, fmt );
	vsprintf( buff, fmt, ap );
	va_end( ap );

	PtSetArg( &args[ 0 ], Pt_ARG_TEXT_STRING, buff, 0 );
	PtSetResources( wgt, 1, args );

	}	// setString


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void getString( const PtWidget_t *wgt, char *string )
	{
	PtArg_t	args[ 1 ];
	char	*value;

	PtSetArg( &args[ 0 ], Pt_ARG_TEXT_STRING, &value, 0 );
	PtGetResources( wgt, 1, args );
	strcpy( string, value );

	}	// getString


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void getPos( const PtWidget_t *wgt, int *x, int *y )
	{
	PtArg_t		args[ 1 ];
	PhPoint_t	*pos;

	PtSetArg( &args[ 0 ], Pt_ARG_POS, &pos, 0 );
	PtGetResources( wgt, 1, args );
	*x = pos->x;
	*y = pos->y;

	}	// getPos


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void setPos( const PtWidget_t *wgt, const int x, const int y )
	{
	PtArg_t		args[ 1 ];
	PhPoint_t	pos;

	pos.x = x;
	pos.y = y;
	PtSetArg( &args[ 0 ], Pt_ARG_POS, &pos, 0 );
	PtSetResources( wgt, 1, args );

	}	// setPos


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void movePos( const PtWidget_t *wgt, const int dx, const int dy )
	{
	PtArg_t		args[ 1 ];
	PhPoint_t	*pos, new_pos;

	PtSetArg( &args[ 0 ], Pt_ARG_POS, &pos, 0 );
	PtGetResources( wgt, 1, args );
	new_pos.x = pos->x + dx;
	new_pos.y = pos->y + dy;
	PtSetArg( &args[ 0 ], Pt_ARG_POS, &new_pos, Pt_DYNAMIC );
	PtSetResources( wgt, 1, args );

	}	// getPos


/*--- -----------------------------------------------------------------	---*/
/*--- Function: setBtn											 		---*/
/*--- 															 		---*/
/*--- Usage:	Put the given button widget into the set/non set state.	---*/
/*--- -----------------------------------------------------------------	---*/
void setBtn( const PtWidget_t *wgt, const BOOL state )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_FLAGS, (state) ? Pt_TRUE : Pt_FALSE, Pt_SET );
	PtSetResources( wgt, 1, args );
	}	// setBtn


/*--- -----------------------------------------------------------------	---*/
/*--- Function: isBtnSet										 		---*/
/*--- 															 		---*/
/*--- Usage:	Is a given button widget into the set state ?			---*/
/*--- -----------------------------------------------------------------	---*/
BOOL isBtnSet( const PtWidget_t *wgt )
	{
	PtArg_t			args[ 1 ];
	unsigned long	*flags;

	PtSetArg( &args[ 0 ], Pt_ARG_FLAGS, &flags, 0 );
	PtGetResources( wgt, 1, args );
	return (BOOL) ((*flags & Pt_SET) == Pt_SET);
	}	// isBtnSet


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void enableBtn( const PtWidget_t *wgt, const BOOL state )
	{
	PtArg_t	args[ 2 ];

	PtSetArg( &args[ 0 ], Pt_ARG_FLAGS, (state) ? Pt_TRUE : Pt_FALSE, Pt_SELECTABLE );
	PtSetArg( &args[ 1 ], Pt_ARG_COLOR, (state) ? Pg_BLACK : Pg_MGREY, 0 );
	PtSetResources( wgt, 2, args );

	}	// enableBtn


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
int ghostBtn( const PtWidget_t *wgt, const BOOL state )
	{
	PtArg_t	args[ 3 ];

	PtSetArg( &args[ 0 ], Pt_ARG_FLAGS, (state) ? Pt_FALSE : Pt_TRUE, Pt_SELECTABLE );
	PtSetArg( &args[ 1 ], Pt_ARG_FLAGS, (state) ? Pt_TRUE : Pt_FALSE, Pt_GHOST );
	PtSetArg( &args[ 2 ], Pt_ARG_FLAGS, (state) ? Pt_TRUE : Pt_FALSE, Pt_BLOCKED );
	PtSetResources( wgt, 3, args );
	return 0;

	}	// ghostBtn


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void selectableBtn( const PtWidget_t *wgt, const BOOL state )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_FLAGS, (state) ? Pt_TRUE : Pt_FALSE, Pt_SELECTABLE );
	PtSetResources( wgt, 1, args );

	}	// selectableBtn


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void setCursorColor( const PtWidget_t *wgt, const PgColor_t color )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_CURSOR_COLOR, color, 0 );
	PtSetResources( wgt, 1, args );

	}	// setCursorColor


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void setCursorType( const PtWidget_t *wgt, int type )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_CURSOR_TYPE, type, 0 );
	PtSetResources( wgt, 1, args );

	}	// setCursorType


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void setBotBorderColor( const PtWidget_t *wgt, const PgColor_t color )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_BOT_BORDER_COLOR, color, 0 );
	PtSetResources( wgt, 1, args );

	}	// setBotBorderColor


// ----------------------------------------------------------------------------
// Add a new line at the end of a list widget
// ----------------------------------------------------------------------------
void addListItem( const PtWidget_t *wgt, const char *fmt, ... )
	{
	va_list			ap;
	char			buff[ 200 ];
	char			*items[ 1 ];
	PtArg_t			args[ 1 ];
	unsigned 		*visible_count, nb_items, top_item_pos;

	/*--- Develop arguments ---*/
	va_start( ap, fmt );
	vsprintf( buff, fmt, ap );
	va_end( ap );

	/*--- Add this new line ---*/
	items[ 0 ] = buff;
	PtListAddItems( wgt, items, 1, 0 );

	/*--- Make this line visible ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_VISIBLE_COUNT, &visible_count, 0 );
	PtGetResources( wgt, 1, args );
	nb_items = getItemsCountList( wgt );
	top_item_pos = nb_items - *visible_count;
	PtSetArg( &args[ 0 ], Pt_ARG_TOP_ITEM_POS, top_item_pos, 0 );
	PtSetResources( wgt, 1, args );
	
	}	// addListItem


// ----------------------------------------------------------------------------
// Selection of a given element into a list
// ----------------------------------------------------------------------------
void selItemSelectedList( const PtWidget_t *wgt, const int item_no )
	{
	USHORT		selected[ 1 ];
	PtArg_t		args[ 1 ];

	selected[ 0 ] = item_no;
	PtSetArg( &args[ 0 ], Pt_ARG_SELECTION_INDEXES, selected, 1 );
	PtSetResources( wgt, 1, args );
	}	// selItemSelectedList
	

// ----------------------------------------------------------------------------
// Which item is currently selected into a list
// ----------------------------------------------------------------------------
int getItemSelectedList( const PtWidget_t *wgt )
	{
	USHORT		*nb_sel;
	USHORT		*selected;
	PtArg_t		args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_SELECTION_INDEXES, &selected, &nb_sel );
	PtGetResources( wgt, 1, args );
	if ( (*nb_sel == 0) || (selected == NULL ) )
		return 0;
	return selected[ 0 ];

	}	// getItemSelectedList
	

// ----------------------------------------------------------------------------
// Number of elements into the list
// ----------------------------------------------------------------------------
int getItemsCountList( const PtWidget_t *wgt )
	{
	USHORT		*nb;
	PtArg_t		args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_LIST_ITEM_COUNT, &nb, 1 );
	PtGetResources( wgt, 1, args );
	return *nb;

	}	// getItemsCountList
	

// ----------------------------------------------------------------------------
// Get the n-th item of a list
// ----------------------------------------------------------------------------
char *getItemList( const PtWidget_t *wgt, const int item_no )
	{
	USHORT		*nb_items;
	char		**items;
	PtArg_t		args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_ITEMS, &items, &nb_items );
	PtGetResources( wgt, 1, args );
	if ( (item_no > *nb_items) || (items == NULL ) )
		return NULL;
	return items[ item_no-1 ];

	}	// getItemList
	

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void setGauge_MinMax( const PtWidget_t *wgt, 
					  const int vmin, const int vmax, const int vcurr )
	{
	PtArg_t	args[ 3 ];

	PtSetArg( &args[ 0 ], Pt_ARG_GAUGE_MINIMUM, vmin, 0 );
	PtSetArg( &args[ 1 ], Pt_ARG_GAUGE_MAXIMUM, vmax, 0 );
	PtSetArg( &args[ 2 ], Pt_ARG_GAUGE_VALUE,   vcurr, 0 );
	PtSetResources( wgt, 3, args );
	PtFlush();

	}	// setGauge_MinMax


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
int getGauge_value( const PtWidget_t *wgt )
	{
	PtArg_t	args[ 1 ];
	int		*value;

	PtSetArg( &args[ 0 ], Pt_ARG_GAUGE_VALUE, &value, 0 );
	PtGetResources( wgt, 1, args );
	return *value;

	}	// getGauge_value


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void setGauge_value( const PtWidget_t *wgt, const int value )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_GAUGE_VALUE, value, 0 );
	PtSetResources( wgt, 1, args );
	PtFlush();

	}	// setGauge_value


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void window_focus( PtWidget_t *wgt )
	{
	PhWindowEvent_t		we;

	/*--- Send appropriate message to the window manager ---*/
	memset( &we, 0, sizeof( PhWindowEvent_t ) );
	we.rid         = PtWidgetRid( wgt );
	we.event_f     = Ph_WM_FOCUS;
	we.event_state = Ph_WM_EVSTATE_FOCUS;
	PtForwardWindowEvent( &we );
	
	}	// window_focus


/*--- -----------------------------------------------------------------	---*/
/*--- Function: getInt													---*/
/*--- 															 		---*/
/*--- Usage:	Read the current numeric value of the given widget.		---*/
/*--- -----------------------------------------------------------------	---*/
int getInt( const PtWidget_t *wgt )
	{
	PtArg_t	args[ 1 ];
	int		*currVal;

	PtSetArg( &args[ 0 ], Pt_ARG_NUMERIC_VALUE, &currVal, 0 );
	PtGetResources( wgt, 1, args );
	return *currVal;

	}	// getInt


/*--- -----------------------------------------------------------------	---*/
/*--- Function: setInt													---*/
/*--- 															 		---*/
/*--- Usage:	Set the current numeric value of the given widget.		---*/
/*--- -----------------------------------------------------------------	---*/
void setInt( const PtWidget_t *wgt, const int val )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_NUMERIC_VALUE, val, 0 );
	PtSetResources( wgt, 1, args );

	}	// setInt


/*--- -----------------------------------------------------------------	---*/
/*--- Function: getDim													---*/
/*--- 															 		---*/
/*--- Usage:	Get the dimension of a widget							---*/
/*--- -----------------------------------------------------------------	---*/
void getDim( const PtWidget_t *wgt, USHORT *w, USHORT *h )
	{
	PtArg_t		args[ 1 ];
	PhDim_t		*dim;

	PtSetArg( &args[ 0 ], Pt_ARG_DIM, &dim, 0 );
	PtGetResources( wgt, 1, args );
	if ( w )
		*w = dim->w;
	if ( h )
		*h = dim->h;

	}	// getDim


/*--- -----------------------------------------------------------------	---*/
/*--- Function: setDim													---*/
/*--- 															 		---*/
/*--- Usage:	Change the dimension of the given widget.				---*/
/*--- -----------------------------------------------------------------	---*/
void setDim( const PtWidget_t *wgt, const int w, const int h )
	{
	PtArg_t		args[ 1 ];
	PhDim_t		size;

	size.w = w;
	size.h = h;
	PtSetArg( &args[ 0 ], Pt_ARG_DIM, &size, 0 );
	PtSetResources( wgt, 1, args );

	}	// setDim


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
int blockWindow( const PtWidget_t *wgt, const BOOL state )
	{
	PtArg_t	args[ 1 ];

	PtSetArg( &args[ 0 ], Pt_ARG_FLAGS, (state) ? Pt_TRUE : Pt_FALSE, Pt_BLOCKED );
	PtSetResources( wgt, 1, args );

	return Pt_CONTINUE;
	}	// blockWindow


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
int busyWindow( const PtWidget_t *wgt, const BOOL state )
	{
	blockWindow( wgt, state );
	setCursorType( wgt, 
		(state) ? Ph_CURSOR_CLOCK : Ph_CURSOR_POINTER );
	PtFlush();
	return Pt_CONTINUE;
	}	// busyWindow


void get_rel_possiz_window( PtWidget_t *wgt, int *x, int *y, int *w, int *h )
	{
	PhArea_t	*area;
	PtArg_t		args[ 1 ];
	PhRect_t	r;

	/*--- Ask for the size and the absolute position ---*/
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &area, 0 );
	PtGetResources( wgt, 1, args );
	*w = area->size.w;
	*h = area->size.h;

	/*--- Translate into relative position to 1st console ---*/
	PhWindowQueryVisible( Ph_QUERY_GRAPHICS, 0, 1, &r );
	*x = area->pos.x % r.ul.x;
	*y = area->pos.y % r.ul.y;

	}	// get_rel_possiz_window


void set_possiz_window( PtWidget_t *wgt, const int x, const int y, const int w, const int h )
	{
	PhArea_t	area;
	PtArg_t		args[ 1 ];
	PhRect_t	r;

	/*--- Translate into relative position to 1st console ---*/
	PhWindowQueryVisible( Ph_QUERY_IG_POINTER, 0, 1, &r );
	area.pos.x = x + r.ul.x;
	area.pos.y = y + r.ul.y;

	/*--- Ask for the size and the absolute position ---*/
	area.size.w = w;
	area.size.h = h;
	PtSetArg( &args[ 0 ], Pt_ARG_AREA, &area, 0 );
	PtSetResources( wgt, 1, args );

	}	// set_possiz_window
